package main

import (
	"flag"
	"log"
	"net/http"
	"os"
)

var mux = http.NewServeMux()

var PublishedImgDir string
var NextImgDir string
var ThumbsDir string

func main() {
	bind := flag.String("bind", ":8080", "Bind port/socket")
	htpasswd_file := flag.String("htpasswd", "", "Admin passwords file, Apache htpasswd format")
	flag.StringVar(&PublishedImgDir, "publishedimgdir", "./images/published/", "Directory where save published pictures")
	flag.StringVar(&NextImgDir, "nextimgdir", "./images/next/", "Directory where save pictures to review")
	flag.StringVar(&ThumbsDir, "thumbsdir", "./images/thumbs/", "Directory where generate thumbs")
	flag.Parse()

	htpasswd := &Htpasswd{}

	if htpasswd_file != nil && *htpasswd_file != "" {
		log.Println("Reading htpasswd file...")
		var err error
		if htpasswd, err = NewHtpasswd(*htpasswd_file); htpasswd == nil {
			log.Fatal("Unable to parse htpasswd:", err)
		}
	} else if NextImgDir == "./images/next/" {
		log.Println("Disable admin interface, images will be published without moderation")
		NextImgDir = PublishedImgDir
	}

	log.Println("Checking paths...")
	if _, err := os.Stat(PublishedImgDir); os.IsNotExist(err) {
		if err := os.MkdirAll(PublishedImgDir, 0755); err != nil {
			log.Fatal(err)
		}
	}
	if _, err := os.Stat(NextImgDir); os.IsNotExist(err) {
		if err := os.MkdirAll(NextImgDir, 0755); err != nil {
			log.Fatal(err)
		}
	}
	if _, err := os.Stat(ThumbsDir); os.IsNotExist(err) {
		if err := os.MkdirAll(ThumbsDir, 0755); err != nil {
			log.Fatal(err)
		}
	}

	log.Println("Registering handlers...")
	authFunc := func(r *http.Request) *User { return Authenticate(*htpasswd, r) }

	if err := sanitizeStaticOptions(); err != nil {
		log.Fatal(err)
	}

	mux.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
		serveStaticAsset(w, r, "index.html")
	})

	mux.Handle("/api/", http.StripPrefix("/api", ApiHandler(authFunc)))

	mux.Handle("/images/", http.StripPrefix("/images", ImagesHandler(
		"images",
		func(*http.Request) bool { return true },
		http.FileServer(http.Dir(PublishedImgDir)),
		GetPublishedImage,
	)))
	mux.Handle("/images/next/", http.StripPrefix("/images/next", ImagesHandler(
		"next",
		func(r *http.Request) bool { return authFunc(r) != nil },
		http.FileServer(http.Dir(NextImgDir)),
		GetNextImage,
	)))

	mux.Handle("/images/thumbs/", http.StripPrefix("/images/thumbs", ImagesHandler(
		"thumbs",
		func(*http.Request) bool { return true },
		http.FileServer(http.Dir(ThumbsDir)),
		GetPublishedImage,
	)))
	mux.Handle("/images/next/thumbs/", http.StripPrefix("/images/next/thumbs", ImagesHandler(
		"nexthumbs",
		func(r *http.Request) bool { return authFunc(r) != nil },
		http.FileServer(http.Dir(ThumbsDir)),
		GetNextImage,
	)))

	mux.HandleFunc("/admin/", func(w http.ResponseWriter, r *http.Request) {
		if authFunc(r) == nil {
			w.Header().Set("WWW-Authenticate", "Basic realm=\"YouP0m\"")
			http.Error(w, "You are not allowed to perform this request.", http.StatusUnauthorized)
		} else {
			serveStaticAsset(w, r, "admin.html")
		}
	})

	log.Println("Ready, listening on", *bind)
	if err := http.ListenAndServe(*bind, mux); err != nil {
		log.Fatal("Unable to listen and serve: ", err)
	}
}
